// ----------------------------------------------------------------------------
// 	Copyright (c) 2001 Thomas E. Hilinski. All rights reserved.
// 	Unauthorized distribution, duplication, sale, and use
//	is prohibited by law.
//
//	This software is made available under the terms of the
//	GNU Lesser General Public License, found at:
//	http://www.gnu.org/licenses/licenses.html#LGPL
//	This copyright notice must remain intact and in place.
// ----------------------------------------------------------------------------
//	Project:  OS-Portable File Classes
//	File:	  TFileList.cpp
//	Class:	  TFileNameList
//
//	Description:
//	Class for generating a list file or path names.
//
// ----------------------------------------------------------------------------
//	Author:	Tom Hilinski, hilinski@lamar.colostate.edu
//	History: See header file.
// ----------------------------------------------------------------------------

#include "TFileList.h"
#include <cstring>

#if ! (defined(_Windows) || defined(__WIN32__) )
// assume UNIX
#include <glob.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif

using namespace TEH;
using namespace std;

// ----------------------------------------------------------------------------
//	static variables
// ----------------------------------------------------------------------------

char const * const TFileNameList::version = "1.20";	// class version string

// ----------------------------------------------------------------------------
//	constructors and destructor
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
//	operator overloads
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
//	public functions
// ----------------------------------------------------------------------------

//	Clear
// 	"clear" data members
void TFileNameList::Clear ()
{
	::DeleteNTCStringList (nameList);
	Initialize ();
}

// ----------------------------------------------------------------------------
//	protected functions
// ----------------------------------------------------------------------------

//	IsValidName
//	Returns true if given name is an valid name, else returns false.
bool TFileNameList::IsValidName (
	char const * const name)
{
	if ( !name )
		return false;

	bool retVal = true;
	const char *badNames[] = { ".", "..", NULL };
	const char **p = badNames;

	while ( *p )
	{
		if ( !strcmp (name, *p) )
		{
			retVal = false;
			break;
		}
		++p;
	}
	return retVal;
}

#if defined(_Windows) || defined(__WIN32__)
//----- Windows version

//	GetFoundName
//	Returns a copy of the file name found in the search.
//	Microsoft Windows version
//	Returns a copy of the file name from the WIN32_FIND_DATA struct
//	filled by calls to the Microsoft Windows functions
//	FindFirstFile and FindNextFile.
inline
char* TFileNameList::GetFoundName (
	WIN32_FIND_DATA& findData )
{
#if defined(__WIN32__)
	// can use long file names
	char* name = 0;
	try
	{
		name = new char [strlen(findData.cFileName) + 1];
		strcpy (name, findData.cFileName);
	}
	catch (...)
	{
		errFlag = true;
	}
#else
	// Windows 3.x - use short file name
	char* name = 0;
	try
	{
		name = new char [strlen(findData.cAlternateFileName) + 1];
		strcpy (name, findData.cAlternateFileName);
	}
	catch (...)
	{
		errFlag = true;
	}
#endif
	return name;
}

//	GetFoundNamePtr
//	Returns the pointer to the file name found in the search.
//	Returns the pointer to the file name from the WIN32_FIND_DATA struct
//	filled by calls to the Microsoft Windows functions
//	FindFirstFile and FindNextFile.
inline
char* TFileNameList::GetFoundNamePtr (
	WIN32_FIND_DATA& findData )
{
#if defined(__WIN32__)
	// can use long file names
	return findData.cFileName;
#else
	// Windows 3.x - use short file name
	return findData.cAlternateFileName;
#endif
}

#endif

//	BuildFileList
//	Builds a NULL-terminated list of file names that match the
//	name mask supplied.
//	Returns false if successful, else true if not.
bool TFileNameList::BuildFileList (
	char const * const mask,
	TFileListType type)
{
	bool retVal = false;

#if defined(_Windows) || defined(__WIN32__)
	// Microsoft Windows version
	// Requires Win32s at a minimum.
	bool addName = false;
	short count = 0;
	WIN32_FIND_DATA findData;
	HANDLE searchHandle;	// returned by FindFirstFile
	BOOL searchResult;	// returned by FindNextFile
	DWORD attr;		// file attributes

	// first count the number of files
	searchHandle = ::FindFirstFile (mask, &findData);
	if ( searchHandle != INVALID_HANDLE_VALUE )
	{
		attr = findData.dwFileAttributes;	// path attributes
		if ( // is a path but not a file
		     ( type != FLT_Normal &&
		       (attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is a file but not a path
		     ( type == FLT_Normal &&
		       !(attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is either type
		     type == FLT_Both
		   )
		{
			if ( IsValidName ( GetFoundNamePtr(findData) ) )
				++count;
		}
	}
	else	// nothing found
	{
		retVal = true;
		goto all_done;
	}
	while ( searchResult = ::FindNextFile (searchHandle, &findData) )
	{
		attr = findData.dwFileAttributes;	// path attributes
		if ( // is a path but not a file
		     ( type != FLT_Normal &&
		       (attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is a file but not a path
		     ( type == FLT_Normal &&
		       !(attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is either type
		     type == FLT_Both
		   )
		{
			if ( IsValidName ( GetFoundNamePtr(findData) ) )
				++count;
		}
	}
	if ( count == 0 )	// nothing found
	{
		retVal = true;
		goto all_done;
	}
	::FindClose (searchHandle);		// reset for reuse
	listSize = count;			// save

	// next, retrieve names
	try
	{
		nameList = new char* [count + 1];  // alloc memory for list
	}
	catch (...)
	{
		errFlag = true;
		retVal = true;
		goto all_done;
 	}
	nameList[count] = NULL;			// null-terminate list
	count = 0;				// zero count for reuse
	// find files again
	searchHandle = ::FindFirstFile (mask, &findData);
	if ( searchHandle != INVALID_HANDLE_VALUE )
	{
		attr = findData.dwFileAttributes;	// path attributes
		if ( // is a path but not a file
		     ( type != FLT_Normal &&
		       (attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is a file but not a path
		     ( type == FLT_Normal &&
		       !(attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is either type
		     type == FLT_Both
		   )
		{
			if ( IsValidName ( GetFoundNamePtr(findData) ) )
				addName = true;
		}
	}
	// save name
	if ( addName )
	{
		nameList[count++] = GetFoundName (findData);
		addName = false;
	}

	while ( count < listSize &&
		(searchResult = ::FindNextFile (searchHandle, &findData)) )
	{
		attr = findData.dwFileAttributes;	// path attributes
		if ( // is a path but not a file
		     ( type != FLT_Normal &&
		       (attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is a file but not a path
		     ( type == FLT_Normal &&
		       !(attr & FILE_ATTRIBUTE_DIRECTORY) ) ||
		     // is either type
		     type == FLT_Both
		   )
		{
			if ( IsValidName ( GetFoundNamePtr(findData) ) )
				addName = true;
		}
		// save name
		if ( addName )
		{
			nameList[count++] = GetFoundName (findData);
			if ( nameList[count - 1] == NULL )	// error?
			{
				retVal = true;
				goto all_done;
			}
			addName = false;
		}
	}
	::FindClose (searchHandle);		// reset for reuse
	all_done:

#else	//----- UNIX version

// make sure PATH_MAX is defined
#ifndef PATH_MAX
#define PATH_MAX 260
#endif

	// glob the path
	glob_t globBuffer;
	int flags = GLOB_ERR;
	int result = glob (mask, flags, NULL, &globBuffer);
	char** p = globBuffer.gl_pathv;
	struct stat statBuf;
	short count = globBuffer.gl_pathc;

	// get memory for list
	// - overallocates needed pointers because we haven't
	//   counted the actual number needed yet.
	try
	{
		nameList = new char* [count + 1];  	// alloc memory
	}
	catch (...)
	{
		errFlag = true;
		retVal = true;
		goto all_done;
	}
	for ( short i = 0; i <= count; ++i )	// null the ptrs
		nameList[i] = 0;

	// collect names of proper type into list
	count = 0;				// reset
	for ( short i = 0; i < globBuffer.gl_pathc; ++i )
	{
	    bool addName = false;
	    int result = stat (*p, &statBuf);
	    if ( result == 0 )	// get stat info
	    {
		if ( // is a path but not a file
		     ( type == FLT_Directory &&
		       S_ISDIR(statBuf.st_mode) ) ||
		     // is a file but not a path
		     ( type == FLT_Normal &&
		       S_ISREG(statBuf.st_mode) ) ||
		     // is either type
		     ( type == FLT_Both &&
		       ( S_ISDIR(statBuf.st_mode) ||
		         S_ISREG(statBuf.st_mode) ) )
		   )
		{
		    if ( IsValidName(*p) )
			addName = true;
		}
		// save name
		if ( addName )
		{
			try
			{
			  nameList[count] = new char [strlen(*p) + 1];
			}
			catch (...)
			{
		    	  errFlag = true;
		    	  retVal = true;
		    	  goto all_done;
		 	}
			strcpy (nameList[count], *p);
			if ( nameList[count] == 0 )	// error?
			{
				retVal = true;
				goto all_done;
			}
			++count;
			addName = false;
		}
	    }
	    ++p;	// next path name
	}
	listSize = count;

	all_done:
	globfree (&globBuffer);

#endif	//----- Windows or UNIX version

	return retVal;
}

// ----------------------------------------------------------------------------
//	private functions
// ----------------------------------------------------------------------------

void TFileNameList::Initialize ()
{
	nameList = NULL;
	listSize = 0;
	errFlag = false;
}

//	Copy
// 	copy to this
void TFileNameList::Copy (
	TFileNameList const & object)
{
	if ( &object )
	{
	    if ( !IsEmpty() )
	    	Clear ();
	    if ( !object.IsEmpty() )
	    {
		// Allocate the list
		try
		{
			listSize = object.listSize;
			::CopyNTCStringList (
				nameList, object.nameList, listSize);
		}
		catch (...)
		{
			errFlag = true;
		}
	    }
	}
}


//--- end of file ---
