#ifndef INC_TFileList_h
#define INC_TFileList_h

// ----------------------------------------------------------------------------
// 	Copyright (c) 2001 Thomas E. Hilinski. All rights reserved.
// 	Unauthorized distribution, duplication, sale, and use
//	is prohibited by law.
//
//	This software is made available under the terms of the
//	GNU Lesser General Public License, found at:
//	http://www.gnu.org/licenses/licenses.html#LGPL
//	This copyright notice must remain intact and in place.
// ----------------------------------------------------------------------------
//	Project:  OS-Portable File Classes
//	File:	  TFileList.h
//	Class:	  TFileNameList
//
//	Description:
//	Class for generating a list file or path names.
// ----------------------------------------------------------------------------
//	Author:	Tom Hilinski, hilinski@lamar.colostate.edu
//	History:
//	Tom Hilinski, Jan03
//	* Version 1.20
//	* Put into namespace TEH.
//	* added a bit more const-correctness.
//	* moved enum TFileListType into class TFileNameList.
//	* changed use of C assert to AssertEx.h
//	* Added public member IsEmpty.
//	* Misc. cleanup.
// 	* Added operator[], copy constructor, operator=
// ----------------------------------------------------------------------------

#if defined(_Windows) || defined(__WIN32__)
#include <windows.h>
#endif

#include "charutil.h"

namespace TEH
{

class TFileNameList
{
public:
	//--- types
	enum TFileListType		// File List types constants
	{
	  FLT_Unknown,			// other or unknown
	  FLT_Normal,			// normal file
	  FLT_Directory,		// directory
	  FLT_Both,			// both normal files and directories
	  FLT_EndOfList			// last item always!
	};

	//--- constructors and destructor
	TFileNameList (
	  char const * const mask,		// file name mask
	  TFileListType type)			// type of file
	  {
	    Initialize ();
	    if ( BuildFileList (mask, type) )
		errFlag = true;
	  }
	~TFileNameList ()
	  {
	    ::DeleteNTCStringList (nameList);
	  }
	TFileNameList (				  // copy constructor
	  TFileNameList const & object)
	  {
	    Initialize ();
	    Copy (object);
	  }

	//--- operators
	TFileNameList& operator= (
	  TFileNameList const & object)
	  {
	    if (this != &object)	// assigning self?
		Copy (object);
	    return *this;
	  }
	char const * const operator[] (		// Get a name in the list
	  short const index			//   index to name list
	  ) const
	  {
#ifndef NDEBUG
	    return (
		nameList && index < listSize ?
		nameList[index] : 0);
#else
	    return nameList[index];
#endif
	  }

	//--- functions
	char const * const * const GetList (	// Returns pointer to list
	  ) const
	  { return nameList; }
	short GetCount () const			// Returns size of list
	  { return listSize; }
	bool IsEmpty () const			// True if empty list
	  { return listSize == 0; }
	bool IsError () const			// Returns true if error
	  { return errFlag; }
	void Clear ();				// Clears the list contents
	static char const * const GetVersion ()	// Get version of this class.
	  { return version; }

protected:
	//--- data
	char** nameList;		// list of file/directory names
	short listSize;			// number of names
	bool errFlag;			// true if error occurred

	//--- functions
	bool BuildFileList (
	  char const * const mask,		// file name mask
	  TFileListType type);			// type of file
	bool IsValidName (
	  char const * const name);
#if defined(_Windows) || defined(__WIN32__)
	// Microsoft Windows version
	char* GetFoundName ( WIN32_FIND_DATA& findData );
	char* GetFoundNamePtr ( WIN32_FIND_DATA& findData );
#endif

private:
	//--- static data
	static char const * const version;	// class version string

	//--- functions
	void Initialize ();
	void Copy (				// Copy to this
	  TFileNameList const & object);

};

}	// namespace TEH

#endif // INC_TFileList_h

