#ifndef INC_CharUtilities_h
#define INC_CharUtilities_h

// ----------------------------------------------------------------------------
//	Copyright (c) 2001-2003 Thomas E. Hilinski. All rights reserved.
//
//	This software is made available under the terms of the
//	GNU Lesser General Public License, found at:
//	http://www.gnu.org/licenses/licenses.html#LGPL
//	This copyright notice must remain intact and in place.
// ----------------------------------------------------------------------------
//	Project:  char and char* utility functions
//	File:	  charutil.h
//
//	Description:
//	Contains a variety of utility functions for char and char* variables.
//	strtrim		Trims the leading and trailing spaces from a string.
//	strpad		Pads string with trailing space to a specified length.
//	strtolower	Converts all chars in a string to lower case.
//	strcmplc	Case-insensitive string comparison.
//	CToCString	Copy with allocation a C string to a C string
//	StringToCString	Copy with allocation a std::string to a C string
//	CopyNTCStringList
// 			Copy with allocation a null-term'd list of C strings.
//	DeleteNTCStringList
//			Deletes memory for a null-terminated list of C strings.
// ----------------------------------------------------------------------------
//	Author:	Tom Hilinski, hilinski@lamar.colostate.edu, Aug2002
//	History:
//	Jan03	Tom Hilinski
//	Added functions CToCString and StringToCString
//	Added CopyNTCStringList, DeleteNTCStringList
// ----------------------------------------------------------------------------
//	This version added to VIDE source package for VIDE version 2.00
//	by Bruce Wampler date: 
// ----------------------------------------------------------------------------

#include <string>
#include <cstring>

char * strtrim (
	char* s);			// string to trim blanks from.

char * strpad (
	char* s, 			// string to pad with blanks at end
	short const sLen);		// length of string after padding

void strtolower (
	char* str);			// string to convert to lower case

int strcmplc (
	char const* str1, 		// first string to compare
	char const* str2);		// second string to compare

inline
char * CToCString (			// Copy with allocate a C string
	char* &toCStr,			//   destination C string pointer
	char const * const fromCStr)	//   source C string
{
	  if ( fromCStr )  // non-destructive
	  {
	    short const fromCStrLength = std::strlen (fromCStr);
	    bool const toCStrLongEnough = ( toCStr ?
		 fromCStrLength <= (short)std::strlen (toCStr) :
		 false );
	    // already allocated?
	    if ( toCStr || !toCStrLongEnough )
	    {
		delete [] toCStr;
		toCStr = 0;
	    }
	    toCStr = new char [fromCStrLength + 1];
	    strcpy ( toCStr, fromCStr );
	  }
	  return toCStr;
}

inline
char * StringToCString (		// Copy with allocate a std::string
	char * toCStr,			//   destination C string
	std::string const & fromStr)	//   source std::string
{
	if ( !fromStr.empty() )
	{
	    if ( toCStr &&
	    	 fromStr.length() > std::strlen (toCStr) )
	    {
		delete [] toCStr;
		toCStr = 0;
		toCStr = new char [fromStr.length() + 1];
	    }
	    strcpy ( toCStr, fromStr.c_str() );
	}
	return toCStr;
}

// Copy with allocation a null-term'd list of C strings.
void CopyNTCStringList (
	char** &toList,				//   destination list
	char const * const * const fromList,	//   source list
	int const listSize);			//   number of C strings

//	DeleteNTCStringList	Deletes memory for a null-terminated list of strings.
void DeleteNTCStringList (
	char** &list);


#endif // INC_CharUtilities_h
